'WebWagon: An HTML Container Class 
'
'Jon Vote - 2004
'
'Idioma Software Inc. 
'www.idioma-software.com
'www.skycoder.com 
'jon@idioma-software.com 
'541-488-1972 (USA)

Imports System.NetC:\Idioma\ITA\CS_GEO_DEM\cn\Operational_Control.html

Imports System.IO
Imports System.Text
Imports System.Text.RegularExpressions

'HTMLPage: HTML Container Class
Public Class HTMLPage

    'Property variables
    Private m_strURL As String = ""
    Private m_strSource As String = ""
    Private m_strHost As String = ""
    Private m_strServerURL As String = ""
    Private m_strPathURL As String = ""
    Private m_strCharacterSet As String = ""
    Private m_strContentEncoding As String = ""
    Private m_lngContentLength As Long = 0
    Private m_strContentType As String = ""
    Private m_strLastModified As String = ""
    Private m_intTimeOutSeconds As Integer = 30
    Private m_bCancel As Boolean = False

    'Feedback events
    Public Event LoadStatus(ByVal URL As String, ByVal Description As String)
    Public Event LoadProgress(ByVal URL As String, ByVal Maximum As Long, _
        ByVal Value As Long)

    Public Property Cancel() As Boolean
        Get
            Cancel = m_bCancel
        End Get
        Set(ByVal Value As Boolean)
            m_bCancel = Value
        End Set
    End Property

    'URL 
    Public ReadOnly Property URL() As String
        Get
            URL = m_strURL
        End Get
    End Property

    'Host: Return the Document's Host name
    Public ReadOnly Property Host() As String
        Get
            Host = m_strHost
        End Get
    End Property

    'ServerURL: Return the Server URL 
    Public ReadOnly Property ServerURL() As String
        Get
            ServerURL = m_strServerURL
        End Get
    End Property

    'PathURL: Return the Path URL 
    Public ReadOnly Property PathURL() As String
        Get
            PathURL = m_strPathURL
        End Get
    End Property

    'Source 
    Public Property Source() As String
        Get
            Source = m_strSource
        End Get
        Set(ByVal Value As String)
            m_strSource = Value
            m_strHost = ""
            m_strCharacterSet = ""
            m_strContentEncoding = ""
            m_lngContentLength = 0
            m_strContentType = ""
            m_strLastModified = ""
        End Set
    End Property

    'CharacterSet 
    Public ReadOnly Property CharacterSet() As String
        Get
            CharacterSet = m_strCharacterSet
        End Get
    End Property

    'ContentLength 
    Public ReadOnly Property ContentLength() As Long
        Get
            ContentLength = m_lngContentLength
        End Get
    End Property

    'ContentType
    Public ReadOnly Property ContentType() As String
        Get
            ContentType = m_strContentType
        End Get
    End Property

    'LastModified
    Public ReadOnly Property LastModified() As String
        Get
            LastModified = m_strLastModified
        End Get
    End Property

    'Head
    Public ReadOnly Property Head() As String
        Get
            Head = GetTagByName("Head", m_strSource)
        End Get
    End Property

    'Title
    Public ReadOnly Property Title() As String
        Get

            'Remove Comments
            Dim strText As String = StripComments(m_strSource)

            'Grab the Title Tag
            strText = GetTagByName("Title", strText)

            'Remove the opening/closing tags
            Dim strPattern As String = "<[^>]*>"
            strText = Regex.Replace(strText, strPattern, "")

            Title = strText
        End Get
    End Property

    'Body
    Public ReadOnly Property Body() As String
        Get
            Dim strBody As String _
             = GetTagByName("Body", m_strSource)
            If strBody = "" Then
                strBody = Me.Source
            End If
            Body = strBody
        End Get
    End Property

    'Text
    Public ReadOnly Property Text() As String
        Get
            Dim r As New Regex("")
            Dim opts As RegexOptions _
              = RegexOptions.IgnoreCase Or RegexOptions.Singleline

            'Start with the Body
            Dim strText As String = Me.Body

            'Remove comments 
            strText = StripComments(strText)

            'Remove any SCRIPT blocks
            Dim strPattern As String _
              = GetExpressionForTagContents("SCRIPT")
            strText = r.Replace(strText, strPattern, "", opts)

            'Remove the remaining tags
            strPattern = "<[^>]*>"
            strText = r.Replace(strText, strPattern, " ", opts)

            'Convert ISO Characters 
            strText = ISOtoASCII(strText)
            strText = Regex.Replace(strText, "&amp;", "&", opts)

            'Remove double white spaces   
            Dim m As MatchCollection
            Do
                strText = r.Replace(strText, "\s\s", " ")
                m = r.Matches(strText, "\s\s")
            Loop Until m.Count = 0

            Text = Trim(strText)

        End Get
    End Property

    'LoadSource: Load a page from the Internet
    Public Function LoadSource(ByVal URL As String) As Boolean

        'Set the URL property var
        m_strURL = URL

        'Make a guess of 40,000 if the content lenght is not not known
        Const DEFAULT_CONTENT_LENGTH As Integer = 40000
        Dim strSource As String = ""
        Dim strHost As String = ""
        Dim strServerURL As String = ""
        Dim strPathURL As String = ""
        Dim strCharacterSet As String = ""
        Dim strContentEncoding As String = ""
        Dim lngContentLength As Long = 0
        Dim strContentType As String = ""
        Dim strServer As String = ""
        Dim strLastModified As String = ""
        Dim lngTotalLength As Long

        'Don't allow blank URL
        If m_strURL = "" Or m_strURL Is Nothing Then
            RaiseEvent LoadStatus(m_strURL, "Error")
            LoadSource = False
        Else
            Try
                'Load the page from the server
                RaiseEvent LoadStatus(m_strURL, "Request")
                RaiseEvent LoadProgress(m_strURL, 0, 0)
                Dim hrqURL As HttpWebRequest _
                  = HttpWebRequest.Create(m_strURL)
                Dim hrspURL As HttpWebResponse = hrqURL.GetResponse()
                Dim srdrInput _
                  As New StreamReader(hrspURL.GetResponseStream())
                Dim chrBuff(255) As Char
                Dim intLen As Integer

                'Get the content length
                If lngContentLength <= 0 Then
                    lngContentLength = DEFAULT_CONTENT_LENGTH
                End If
                RaiseEvent LoadStatus(m_strURL, "Load")

                'Set the timeout
                Dim tmeExpire As DateTime _
                  = DateAdd(DateInterval.Second, _
                    m_intTimeOutSeconds, Now)

                'Loop until the page is loaded
                Do
                    If m_bCancel Then
                        LoadSource = False
                        Exit Function
                    End If
                    intLen = srdrInput.Read(chrBuff, 0, 256)
                    Dim strBuff As New String(chrBuff, 0, intLen)
                    strSource = strSource & strBuff
                    lngTotalLength = lngTotalLength + intLen
                    If lngTotalLength > lngContentLength Then
                        lngContentLength = 2 * lngTotalLength
                    End If
                    RaiseEvent LoadProgress _
                      (m_strURL, lngContentLength, lngTotalLength)
                    If DateDiff(DateInterval.Second, tmeExpire, Now) _
                      > 0 Then
                        RaiseEvent LoadStatus(m_strURL, "Error")
                        LoadSource = False
                        Exit Do
                    End If
                Loop While intLen
                srdrInput.Close()
                hrspURL.Close()
                With hrspURL
                    strHost = .ResponseUri.Host

                    'URL to current path 
                    Dim m As Match _
                        = Regex.Match(.ResponseUri.AbsoluteUri, _
                          "/", RegexOptions.RightToLeft)
                    If m Is Nothing Then
                        strPathURL = .ResponseUri.AbsoluteUri & "/"
                    Else
                        strPathURL _
                         = .ResponseUri.AbsoluteUri.Substring(0, m.Index) & "/"
                    End If

                    'URL to server
                    m = Regex.Match(.ResponseUri.AbsoluteUri, _
                          strHost, RegexOptions.RightToLeft _
                            Or RegexOptions.IgnoreCase)
                    If m Is Nothing Then
                        strServerURL = .ResponseUri.AbsoluteUri
                    Else
                        strServerURL _
                         = .ResponseUri.AbsoluteUri.Substring _
                           (0, m.Index + strHost.Length)
                    End If

                    strCharacterSet = .CharacterSet
                    strContentEncoding = .ContentEncoding
                    lngContentLength = .ContentLength
                    strContentType = .ContentType
                    strLastModified = .LastModified
                End With
                RaiseEvent LoadStatus(m_strURL, "Complete")
                LoadSource = True
            Catch
                RaiseEvent LoadStatus(m_strURL, "Error")
                LoadSource = False
            End Try
        End If

        'Set the property vars
        m_strHost = strHost
        m_strServerURL = strServerURL
        m_strPathURL = strPathURL
        m_strSource = strSource
        m_strCharacterSet = strCharacterSet
        m_strContentEncoding = strContentEncoding
        m_lngContentLength = lngContentLength
        m_strContentType = strContentType
        m_strLastModified = strLastModified

        'Done
        RaiseEvent LoadProgress(m_strURL, lngTotalLength, lngTotalLength)

    End Function

    'GetTagsByName: Returns all matches for TagName
    Public Function GetTagsByName(ByVal TagName As String) As String()
        GetTagsByName = GetTagsByName(TagName, m_strSource)
    End Function
    Public Function GetTagsByName(ByVal TagName As String, _
      ByVal Source As String) As String()
        Dim r As New Regex("")
        Dim opts As RegexOptions _
          = RegexOptions.IgnoreCase Or RegexOptions.Singleline
        Dim strPattern As String

        'Normally we will ignore comments
        If TagName <> "!" Then
            Source = StripComments(Source)
            strPattern = "<(?<TagName>" & TagName & ")(>|\s+[^>]*>)"
        Else
            strPattern = "<(?<TagName>" + TagName + ")--"
        End If

        'Find the beginning position of each matching tag
        Dim mc As MatchCollection _
            = r.Matches(Source, strPattern, opts)

        'Grab the contents of each matching tag 
        Dim m As Match
        Dim strTagContents() As String
        Dim intIndex As Integer = 0
        For Each m In mc
            ReDim Preserve strTagContents(intIndex)
            strTagContents(intIndex) _
              = GetTagByName(TagName, _
              Mid(Source, m.Groups("TagName").Index))
            intIndex = intIndex + 1
        Next
        GetTagsByName = strTagContents
    End Function

    'GetHRefs: Return all HRefs
    Public Function GetHRefs() As String()
        Dim strSource As String = StripComments(m_strSource)
        Dim r As New Regex( _
          "<a[^>]*href\s*=\s*""?(?<HRef>[^"">\s]*)""?[^>]*>", _
           RegexOptions.IgnoreCase Or RegexOptions.Singleline)
        Dim mc As MatchCollection _
            = r.Matches(Source)
        Dim m As Match
        Dim intIndex As Integer = 0
        Dim strHRefs() As String
        For Each m In mc
            Dim strHRef = Trim(m.Groups("HRef").Value)
            If strHRef <> "" Then
                'Normalize the href
                If Left(strHRef, 1) <> "#" Then
                    If Left(strHRef, 1) = "/" Then
                        strHRef = m_strServerURL & strHRef
                    ElseIf StrComp(Left(strHRef, 7), "http://", _
                        CompareMethod.Text) <> 0 Then
                        If StrComp(Left(strHRef, 3), "www", _
                         CompareMethod.Text) = 0 Then
                            strHRef = "http://" & strHRef
                        Else
                            strHRef = m_strPathURL & strHRef
                        End If
                    End If
                End If
            End If
            ReDim Preserve strHRefs(intIndex)
            strHRefs(intIndex) = strHRef
            intIndex = intIndex + 1
        Next

        'Return the list
        GetHRefs = strHRefs
    End Function

    'GetAllTags: Return all tags in document.
    Public Function GetAllTags() As String()
        Dim strPatternTag = "<(?<Comment>!?)(?<Tag>[^>/\s]+)(>|\s+[^>]*>)"
        Dim r As Regex
        Dim mc As MatchCollection _
          = r.Matches(m_strSource, strPatternTag, _
            RegexOptions.IgnoreCase.IgnoreCase _
             Or RegexOptions.Singleline)
        Dim intLength As Integer = m_strSource.Length
        Dim i As Integer
        Dim strTagList() As String
        Dim strTagName As String
        Dim strTagData As String
        For i = 0 To mc.Count - 1
            If m_bCancel Then
                GetAllTags = Nothing
                Exit Function
            End If
            'Get the tag name from the capture
            If mc(i).Groups("Comment").Value = "" Then
                strTagName = mc(i).Groups("Tag").Value
            Else
                strTagName = "!"
            End If

            'Find the Ending Tag starting at the current position 
            strTagData = GetTagByName(strTagName, _
                m_strSource.Substring(mc(i).Groups("Tag").Index - 1))

            ReDim Preserve strTagList(i)
            strTagList(i) = strTagData
        Next
        GetAllTags = strTagList
    End Function

    'GetExpressionForTagContents: Returns Regular  
    'Expression to extract a given tag's contents.
    Private Function GetExpressionForTagContents _
      (ByVal strTagName As String) As String
        Dim strPatternTag As String
        If strTagName = "!" Then
            strPatternTag = "<!.*?-->"
        ElseIf StrComp(strTagName, "!doctype", CompareMethod.Text) _
            = 0 Then
            strPatternTag = "<!doctype.*?>"
        ElseIf StrComp(strTagName, "br", CompareMethod.Text) = 0 Then
            strPatternTag = "<br\s*/?\s*>"
        Else
            strPatternTag _
              = "<(" & strTagName & ")(>|\s+[^>]*>).*?</\1\s*>"
        End If
        GetExpressionForTagContents = strPatternTag
    End Function

    'GetTagByName: Assumes tag exists
    'Returns everything up to closing tag or 
    'to first < if no closing tag found
    Private Function GetTagByName _
        (ByVal strTagName As String, _
          ByVal strSource As String) As String
        Dim strPatternTag As String = _
          GetExpressionForTagContents(strTagName)
        Dim strPatternTagNoClose = "<" _
          & strTagName & "(>|\s+[^>]*>)[^<]"
        Dim r As New Regex("")
        Dim opts As RegexOptions _
          = RegexOptions.IgnoreCase _
            Or RegexOptions.Singleline
        Dim m As Match
        Dim mc As MatchCollection
        Dim strGetTagByName As String

        m = r.Match(strSource, strPatternTag, opts)
        If m.Value = "" Then
            m = r.Match(strSource, strPatternTagNoClose, opts)
            If m Is Nothing Then
                strGetTagByName = strSource
            Else
                strGetTagByName = m.Value
            End If
        Else
            strGetTagByName = m.Value
        End If
        GetTagByName = strGetTagByName
    End Function

    'StripComments: Return Source minus comments
    Private Function StripComments(ByVal strSource As String) As String
        Dim r As New Regex( _
          GetExpressionForTagContents("!"))
        StripComments = r.Replace(strSource, "")
    End Function

    'ISOtoASCII: Convert ISO Characters to ASCII
    Private Function ISOtoASCII(ByVal strText As String) As String

        Dim strFrom(130) As String
        Dim strTo(130) As String

        strFrom(1) = "&Agrave;" : strTo(1) = "" 'capital A, grave accent       
        strFrom(2) = "&agrave;" : strTo(2) = "" 'small a, grave accent        
        strFrom(3) = "&Aacute;" : strTo(3) = "" 'capital A, acute accent      
        strFrom(4) = "&aacute;" : strTo(4) = "" 'small a, acute accent        
        strFrom(5) = "&Acirc;" : strTo(5) = "" 'capital A, circumflex        
        strFrom(6) = "&acirc;" : strTo(6) = "" 'small a, circumflex          
        strFrom(7) = "&Atilde;" : strTo(7) = "" 'capital A, tilde             
        strFrom(8) = "&atilde;" : strTo(8) = "" 'small a, tilde               
        strFrom(9) = "&Auml;" : strTo(9) = "" 'capital A, diresis/umlaut   
        strFrom(10) = "&auml;" : strTo(10) = "" 'small a, diresis/umlaut     
        strFrom(11) = "&Aring;" : strTo(11) = "" 'capital A, ring              
        strFrom(12) = "&aring;" : strTo(12) = "" 'small a, ring                
        strFrom(13) = "&AElig;" : strTo(13) = "" 'capital AE ligature          
        strFrom(14) = "&aelig;" : strTo(14) = "" 'small ae ligature            
        strFrom(15) = "&Ccedil;" : strTo(15) = "" 'capital C, cedilla           
        strFrom(16) = "&ccedil;" : strTo(16) = "" 'small c, cedilla             
        strFrom(17) = "&Egrave;" : strTo(17) = "" 'capital E, grave accent      
        strFrom(18) = "&egrave;" : strTo(18) = "" 'small e, grave accent        
        strFrom(19) = "&Eacute;" : strTo(19) = "" 'capital E, acute accent      
        strFrom(20) = "&eacute;" : strTo(20) = "" 'small e, acute accent        
        strFrom(21) = "&Ecirc;" : strTo(21) = "" 'capital E, circumflex        
        strFrom(22) = "&ecirc;" : strTo(22) = "" 'small e, circumflex          
        strFrom(23) = "&Euml;" : strTo(23) = "" 'capital E, diresis/umlaut   
        strFrom(24) = "&euml;" : strTo(24) = "" 'small e, diresis/umlaut     
        strFrom(25) = "&Igrave;" : strTo(25) = "" 'capital I, grave accent      
        strFrom(26) = "&igrave;" : strTo(26) = "" 'small i, grave accent        
        strFrom(27) = "&Iacute;" : strTo(27) = "" 'capital I, acute accent      
        strFrom(28) = "&iacute;" : strTo(28) = "" 'small i, acute accent        
        strFrom(29) = "&Icirc;" : strTo(29) = "" 'capital I, circumflex        
        strFrom(30) = "&icirc;" : strTo(30) = "" 'small i, circumflex          
        strFrom(31) = "&Iuml;" : strTo(31) = "" 'capital I, diresis/umlaut   
        strFrom(32) = "&iuml;" : strTo(32) = "" 'small i, diresis/umlaut  
        strFrom(33) = "&ETH;" : strTo(33) = "" 'capital Eth, Icelandic
        strFrom(34) = "&eth;" : strTo(34) = "" 'small eth, Icelandic
        strFrom(35) = "&Ntilde;" : strTo(35) = "" 'capital N, tilde        
        strFrom(36) = "&ntilde;" : strTo(36) = "" 'small n, tilde               
        strFrom(37) = "&Ograve;" : strTo(37) = "" 'capital O, grave accent      
        strFrom(38) = "&ograve;" : strTo(38) = "" 'small o, grave accent             
        strFrom(39) = "&Oacute;" : strTo(39) = "" 'capital O, acute accent      
        strFrom(40) = "&oacute;" : strTo(40) = "" 'small o, acute accent        
        strFrom(41) = "&Ocirc;" : strTo(41) = "" 'capital O, circumflex   
        strFrom(42) = "&ocirc;" : strTo(42) = "" 'small o, circumflex            
        strFrom(43) = "&Otilde;" : strTo(43) = "" 'capital O, tilde             
        strFrom(44) = "&otilde;" : strTo(44) = "" 'small o, tilde               
        strFrom(45) = "&Ouml;" : strTo(45) = "" 'capital O, diresis/umlaut 
        strFrom(46) = "&ouml;" : strTo(46) = "" 'small o, diresis/umlaut   
        strFrom(47) = "&Oslash;" : strTo(47) = "" 'capital O, slash                   
        strFrom(48) = "&oslash;" : strTo(48) = "" 'small o, slash          
        strFrom(49) = "&Ugrave;" : strTo(49) = "" 'capital U, grave accent           
        strFrom(50) = "&ugrave;" : strTo(50) = "" 'small u, grave accent        
        strFrom(51) = "&Uacute;" : strTo(51) = "" 'capital U, acute accent      
        strFrom(52) = "&uacute;" : strTo(52) = "" 'small u, acute accent        
        strFrom(53) = "&Ucirc;" : strTo(53) = "" 'capital U, circumflex          
        strFrom(54) = "&ucirc;" : strTo(54) = "" 'small u, circumflex            
        strFrom(55) = "&Uuml;" : strTo(55) = "" 'capital U, diresis/umlaut 
        strFrom(56) = "&uuml;" : strTo(56) = "" 'small u, diresis/umlaut      
        strFrom(57) = "&Yacute;" : strTo(57) = "" 'capital Y, acute accent      
        strFrom(58) = "&yacute;" : strTo(58) = "" 'small y, acute accent        
        strFrom(59) = "&THORN;" : strTo(59) = "" 'capital Thorn, Icelandic       
        strFrom(60) = "&thorn;" : strTo(60) = "" 'small thorn, Icelandic         
        strFrom(61) = "&szlig;" : strTo(61) = "" 'small sharp s, German sz           
        strFrom(62) = "&yuml;" : strTo(62) = "" 'small y, diresis/umlaut 
        strFrom(63) = "&nbsp;" : strTo(63) = " " 'non-breaking space          
        strFrom(64) = "&iexcl;" : strTo(64) = "" 'inverted exclamation mark   
        strFrom(65) = "&cent;" : strTo(65) = "" 'cent sign                   
        strFrom(66) = "&pound;" : strTo(66) = "" 'pound sign                  
        strFrom(67) = "&curren;" : strTo(67) = "" 'general currency sign       
        strFrom(68) = "&yen;" : strTo(68) = "" 'yen sign                    
        strFrom(69) = "&brvbar;" : strTo(69) = "" 'broken (vertical) bar       
        strFrom(70) = "&sect;" : strTo(70) = "" 'section sign                
        strFrom(71) = "&uml;" : strTo(71) = "" 'umlaut/dieresis             
        strFrom(72) = "&copy;" : strTo(72) = "" 'copyright sign              
        strFrom(73) = "&ordf;" : strTo(73) = "" 'ordinal indicator, fem      
        strFrom(74) = "&laquo;" : strTo(74) = "" 'angle quotation mark, left  
        strFrom(75) = "&not;" : strTo(75) = "" 'not sign                    
        strFrom(76) = "&shy;" : strTo(76) = "" 'soft hyphen                 
        strFrom(77) = "&reg;" : strTo(77) = "" 'registered sign             
        strFrom(78) = "&macr;" : strTo(78) = "" 'macron                      
        strFrom(79) = "&deg;" : strTo(79) = "" 'degree sign                 
        strFrom(80) = "&#160;" : strTo(80) = " " 'non-breaking space          
        strFrom(81) = "&#161;" : strTo(81) = "" 'inverted exclamation mark   
        strFrom(82) = "&#162;" : strTo(82) = "" 'cent sign                   
        strFrom(83) = "&#163;" : strTo(83) = "" 'pound sign                  
        strFrom(84) = "&#164;" : strTo(84) = "" 'general currency sign       
        strFrom(85) = "&#165;" : strTo(85) = "" 'yen sign                    
        strFrom(86) = "&#166;" : strTo(86) = "" 'broken (vertical) bar       
        strFrom(87) = "&#167;" : strTo(87) = "" 'section sign                
        strFrom(88) = "&#168;" : strTo(88) = "" 'umlaut/dieresis             
        strFrom(89) = "&#169;" : strTo(89) = "" 'copyright sign              
        strFrom(90) = "&#170;" : strTo(90) = "" 'ordinal indicator, fem      
        strFrom(91) = "&#171;" : strTo(91) = "" 'angle quotation mark, left  
        strFrom(92) = "&#172;" : strTo(92) = "" 'not sign                    
        strFrom(93) = "&#173;" : strTo(93) = "" 'soft hyphen                 
        strFrom(94) = "&#174;" : strTo(94) = "" 'registered sign             
        strFrom(95) = "&#175;" : strTo(95) = "" 'macron                      
        strFrom(96) = "&#176;" : strTo(96) = "" 'degree sign                 
        strFrom(97) = "&plusmn;" : strTo(97) = "" 'plus-or-minus sign          
        strFrom(98) = "&sup2;" : strTo(98) = "" 'superscript two          
        strFrom(99) = "&sup3;" : strTo(99) = "" 'superscript three        
        strFrom(100) = "&acute;" : strTo(100) = "" 'acute accent             
        strFrom(101) = "&micro;" : strTo(101) = "" 'micro sign                
        strFrom(102) = "&para;" : strTo(102) = "" 'pilcrow (paragraph sign) 
        strFrom(103) = "&middot;" : strTo(103) = "" 'middle dot               
        strFrom(104) = "&cedil;" : strTo(104) = "" 'cedilla                  
        strFrom(105) = "&sup1;" : strTo(105) = "" 'superscript one          
        strFrom(106) = "&ordm;" : strTo(106) = "" 'ordinal indicator, male  
        strFrom(107) = "&raquo;" : strTo(107) = "" 'angle quotation mark, right   
        strFrom(108) = "&frac14;" : strTo(108) = "" 'fraction one-quarter          
        strFrom(109) = "&frac12;" : strTo(109) = "" 'fraction one-half             
        strFrom(110) = "&frac34;" : strTo(110) = "" 'fraction three-quarters       
        strFrom(111) = "&iquest;" : strTo(111) = "" 'inverted question mark        
        strFrom(112) = "&times;" : strTo(112) = "" 'multiply sign                 
        strFrom(113) = "&div;" : strTo(113) = "" 'division sign             
        strFrom(114) = "&#177;" : strTo(114) = "" 'plus-or-minus sign          
        strFrom(115) = "&#178;" : strTo(115) = "" 'superscript two          
        strFrom(116) = "&#179;" : strTo(116) = "" 'superscript three        
        strFrom(117) = "&#180;" : strTo(117) = "" 'acute accent             
        strFrom(118) = "&#181;" : strTo(118) = "" 'micro sign                
        strFrom(119) = "&#182;" : strTo(119) = "" 'pilcrow (paragraph sign) 
        strFrom(120) = "&#183;" : strTo(120) = "" 'middle dot               
        strFrom(121) = "&#184;" : strTo(121) = "" 'cedilla                  
        strFrom(122) = "&#185;" : strTo(122) = "" 'superscript one          
        strFrom(123) = "&#186;" : strTo(123) = "" 'ordinal indicator, male  
        strFrom(124) = "&#187;" : strTo(124) = "" 'angle quotation mark, right   
        strFrom(125) = "&#188;" : strTo(125) = "" 'fraction one-quarter          
        strFrom(126) = "&#189;" : strTo(126) = "" 'fraction one-half             
        strFrom(127) = "&#190;" : strTo(127) = "" 'fraction three-quarters       
        strFrom(128) = "&#191;" : strTo(128) = "" 'inverted question mark        
        strFrom(129) = "&#215;" : strTo(129) = "" 'multiply sign                 
        strFrom(130) = "&#247;" : strTo(130) = "" 'division sign             

        Dim i As Integer
        Dim r As Regex
        For i = 1 To strFrom.Length - 1
            strText = r.Replace(strText, strFrom(i), strTo(i), _
             RegexOptions.IgnoreCase.IgnoreCase _
              Or RegexOptions.Singleline.Singleline)
        Next
        ISOtoASCII = strText

    End Function

    Public Sub Dispose()
        m_bCancel = True
    End Sub
End Class